@@ -52,6 +52,12 @@
 
 static struct udev_monitor *udev_monitor;
 
+#ifdef CONFIG_UDEV_KMS
+static Bool
+config_udev_odev_setup_attribs(const char *path, const char *syspath,
+                               config_odev_probe_proc_ptr probe_callback);
+#endif
+
 static void
 device_added(struct udev_device *udev_device)
 {
@@ -61,7 +67,7 @@ device_added(struct udev_device *udev_device)
     const char *tags_prop;
     const char *key, *value, *tmp;
     InputOption *input_options;
-    InputAttributes attrs = {};
+    InputAttributes attrs = { };
     DeviceIntPtr dev = NULL;
     struct udev_list_entry *set, *entry;
     struct udev_device *parent;
@@ -85,11 +91,24 @@ device_added(struct udev_device *udev_device)
     if (!SeatId && strcmp(dev_seat, "seat0"))
         return;
 
+#ifdef CONFIG_UDEV_KMS
+    if (!strcmp(udev_device_get_subsystem(udev_device), "drm")) {
+        const char *sysname = udev_device_get_sysname(udev_device);
+
+        if (strncmp(sysname, "card", 4) != 0)
+            return;
+
+        LogMessage(X_INFO, "config/udev: Adding drm device (%s)\n", path);
+
+        config_udev_odev_setup_attribs(path, syspath, NewGPUDeviceRequest);
+        return;
+    }
+#endif
+
     if (!udev_device_get_property_value(udev_device, "ID_INPUT")) {
         LogMessageVerb(X_INFO, 10,
                        "config/udev: ignoring device %s without "
-                       "property ID_INPUT set\n",
-                       path);
+                       "property ID_INPUT set\n", path);
         return;
     }
 
@@ -116,7 +135,8 @@ device_added(struct udev_device *udev_device)
         LOG_SYSATTR(ppath, "id", pnp_id);
 
         /* construct USB ID in lowercase hex - "0000:ffff" */
-        if (product && sscanf(product, "%*x/%4x/%4x/%*x", &usb_vendor, &usb_model) == 2) {
+        if (product &&
+            sscanf(product, "%*x/%4x/%4x/%*x", &usb_vendor, &usb_model) == 2) {
             if (asprintf(&attrs.usb_id, "%04x:%04x", usb_vendor, usb_model)
                 == -1)
                 attrs.usb_id = NULL;
@@ -145,7 +165,7 @@ device_added(struct udev_device *udev_device)
 
     if (device_is_duplicate(config_info)) {
         LogMessage(X_WARNING, "config/udev: device %s already added. "
-                              "Ignoring.\n", name);
+                   "Ignoring.\n", name);
         goto unwind;
     }
 
@@ -155,39 +175,50 @@ device_added(struct udev_device *udev_device)
         if (!key)
             continue;
         value = udev_list_entry_get_value(entry);
-        if (!strncasecmp(key, UDEV_XKB_PROP_KEY,
-                                sizeof(UDEV_XKB_PROP_KEY) - 1)) {
+        if (!strncasecmp(key, UDEV_XKB_PROP_KEY, sizeof(UDEV_XKB_PROP_KEY) - 1)) {
             LOG_PROPERTY(path, key, value);
             tmp = key + sizeof(UDEV_XKB_PROP_KEY) - 1;
             if (!strcasecmp(tmp, "rules"))
-                input_options = input_option_new(input_options, "xkb_rules", value);
+                input_options =
+                    input_option_new(input_options, "xkb_rules", value);
             else if (!strcasecmp(tmp, "layout"))
-                input_options = input_option_new(input_options, "xkb_layout", value);
+                input_options =
+                    input_option_new(input_options, "xkb_layout", value);
             else if (!strcasecmp(tmp, "variant"))
-                input_options = input_option_new(input_options, "xkb_variant", value);
+                input_options =
+                    input_option_new(input_options, "xkb_variant", value);
             else if (!strcasecmp(tmp, "model"))
-                input_options = input_option_new(input_options, "xkb_model", value);
+                input_options =
+                    input_option_new(input_options, "xkb_model", value);
             else if (!strcasecmp(tmp, "options"))
-                input_options = input_option_new(input_options, "xkb_options", value);
-        } else if (!strcmp(key, "ID_VENDOR")) {
+                input_options =
+                    input_option_new(input_options, "xkb_options", value);
+        }
+        else if (!strcmp(key, "ID_VENDOR")) {
             LOG_PROPERTY(path, key, value);
             attrs.vendor = strdup(value);
-        } else if (!strcmp(key, "ID_INPUT_KEY")) {
+        }
+        else if (!strcmp(key, "ID_INPUT_KEY")) {
             LOG_PROPERTY(path, key, value);
             attrs.flags |= ATTR_KEYBOARD;
-        } else if (!strcmp(key, "ID_INPUT_MOUSE")) {
+        }
+        else if (!strcmp(key, "ID_INPUT_MOUSE")) {
             LOG_PROPERTY(path, key, value);
             attrs.flags |= ATTR_POINTER;
-        } else if (!strcmp(key, "ID_INPUT_JOYSTICK")) {
+        }
+        else if (!strcmp(key, "ID_INPUT_JOYSTICK")) {
             LOG_PROPERTY(path, key, value);
             attrs.flags |= ATTR_JOYSTICK;
-        } else if (!strcmp(key, "ID_INPUT_TABLET")) {
+        }
+        else if (!strcmp(key, "ID_INPUT_TABLET")) {
             LOG_PROPERTY(path, key, value);
             attrs.flags |= ATTR_TABLET;
-        } else if (!strcmp(key, "ID_INPUT_TOUCHPAD")) {
+        }
+        else if (!strcmp(key, "ID_INPUT_TOUCHPAD")) {
             LOG_PROPERTY(path, key, value);
             attrs.flags |= ATTR_TOUCHPAD;
-        } else if (!strcmp(key, "ID_INPUT_TOUCHSCREEN")) {
+        }
+        else if (!strcmp(key, "ID_INPUT_TOUCHSCREEN")) {
             LOG_PROPERTY(path, key, value);
             attrs.flags |= ATTR_TOUCHSCREEN;
         }
@@ -212,6 +243,7 @@ device_added(struct udev_device *udev_device)
     free(attrs.vendor);
     if (attrs.tags) {
         char **tag = attrs.tags;
+
         while (*tag) {
             free(*tag);
             tag++;
@@ -228,6 +260,22 @@ device_removed(struct udev_device *device)
     char *value;
     const char *syspath = udev_device_get_syspath(device);
 
+#ifdef CONFIG_UDEV_KMS
+    if (!strcmp(udev_device_get_subsystem(device), "drm")) {
+        const char *sysname = udev_device_get_sysname(device);
+        const char *path = udev_device_get_devnode(device);
+
+        if (strncmp(sysname,"card", 4) != 0)
+            return;
+        ErrorF("removing GPU device %s %d\n", syspath, path);
+        if (!path)
+            return;
+
+        config_udev_odev_setup_attribs(path, syspath, DeleteGPUDeviceRequest);
+        return;
+    }
+#endif
+
     if (asprintf(&value, "udev:%s", syspath) == -1)
         return;
 
@@ -246,7 +294,7 @@ wakeup_handler(pointer data, int err, pointer read_mask)
     if (err < 0)
         return;
 
-    if (FD_ISSET(udev_fd, (fd_set *)read_mask)) {
+    if (FD_ISSET(udev_fd, (fd_set *) read_mask)) {
         udev_device = udev_monitor_receive_device(udev_monitor);
         if (!udev_device)
             return;
@@ -268,6 +316,39 @@ block_handler(pointer data, struct timeval **tv, pointer read_mask)
 {
 }
 
+int
+config_udev_pre_init(void)
+{
+    struct udev *udev;
+
+    udev = udev_new();
+    if (!udev)
+        return 0;
+
+    udev_monitor = udev_monitor_new_from_netlink(udev, "udev");
+    if (!udev_monitor)
+        return 0;
+
+    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "input",
+                                                    NULL);
+    /* For Wacom serial devices */
+    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "tty", NULL);
+#ifdef CONFIG_UDEV_KMS
+    /* For output GPU devices */
+    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "drm", NULL);
+#endif
+
+#ifdef HAVE_UDEV_MONITOR_FILTER_ADD_MATCH_TAG
+    if (SeatId && strcmp(SeatId, "seat0"))
+        udev_monitor_filter_add_match_tag(udev_monitor, SeatId);
+#endif
+    if (udev_monitor_enable_receiving(udev_monitor)) {
+        ErrorF("config/udev: failed to bind the udev monitor\n");
+        return 0;
+    }
+    return 1;
+}
+
 int
 config_udev_init(void)
 {
@@ -275,32 +356,16 @@ config_udev_init(void)
     struct udev_enumerate *enumerate;
     struct udev_list_entry *devices, *device;
 
-    udev = udev_new();
-    if (!udev)
-        return 0;
-    udev_monitor = udev_monitor_new_from_netlink(udev, "udev");
-    if (!udev_monitor)
-        return 0;
-
-    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "input", NULL);
-    udev_monitor_filter_add_match_subsystem_devtype(udev_monitor, "tty", NULL); /* For Wacom serial devices */
-
-#ifdef HAVE_UDEV_MONITOR_FILTER_ADD_MATCH_TAG
-    if (SeatId && strcmp(SeatId, "seat0"))
-        udev_monitor_filter_add_match_tag(udev_monitor, SeatId);
-#endif
-
-    if (udev_monitor_enable_receiving(udev_monitor)) {
-        ErrorF("config/udev: failed to bind the udev monitor\n");
-        return 0;
-    }
-
+    udev = udev_monitor_get_udev(udev_monitor);
     enumerate = udev_enumerate_new(udev);
     if (!enumerate)
         return 0;
 
     udev_enumerate_add_match_subsystem(enumerate, "input");
     udev_enumerate_add_match_subsystem(enumerate, "tty");
+#ifdef CONFIG_UDEV_KMS
+    udev_enumerate_add_match_subsystem(enumerate, "drm");
+#endif
 
 #ifdef HAVE_UDEV_ENUMERATE_ADD_MATCH_TAG
     if (SeatId && strcmp(SeatId, "seat0"))
@@ -311,7 +376,8 @@ config_udev_init(void)
     devices = udev_enumerate_get_list_entry(enumerate);
     udev_list_entry_foreach(device, devices) {
         const char *syspath = udev_list_entry_get_name(device);
-        struct udev_device *udev_device = udev_device_new_from_syspath(udev, syspath);
+        struct udev_device *udev_device =
+            udev_device_new_from_syspath(udev, syspath);
 
         /* Device might be gone by the time we try to open it */
         if (!udev_device)
@@ -344,3 +410,72 @@ config_udev_fini(void)
     udev_monitor = NULL;
     udev_unref(udev);
 }
+
+#ifdef CONFIG_UDEV_KMS
+
+static Bool
+config_udev_odev_setup_attribs(const char *path, const char *syspath,
+                               config_odev_probe_proc_ptr probe_callback)
+{
+    struct OdevAttributes *attribs = config_odev_allocate_attribute_list();
+    int ret;
+
+    if (!attribs)
+        return FALSE;
+
+    ret = config_odev_add_attribute(attribs, ODEV_ATTRIB_PATH, path);
+    if (ret == FALSE)
+        goto fail;
+
+    ret = config_odev_add_attribute(attribs, ODEV_ATTRIB_SYSPATH, syspath);
+    if (ret == FALSE)
+        goto fail;
+
+    /* ownership of attribs is passed to probe layer */
+    probe_callback(attribs);
+    return TRUE;
+fail:
+    config_odev_free_attributes(attribs);
+    free(attribs);
+    return FALSE;
+}
+
+void
+config_udev_odev_probe(config_odev_probe_proc_ptr probe_callback)
+{
+    struct udev *udev;
+    struct udev_enumerate *enumerate;
+    struct udev_list_entry *devices, *device;
+
+    udev = udev_monitor_get_udev(udev_monitor);
+    enumerate = udev_enumerate_new(udev);
+    if (!enumerate)
+        return;
+
+    udev_enumerate_add_match_subsystem(enumerate, "drm");
+    udev_enumerate_add_match_sysname(enumerate, "card[0-9]*");
+    udev_enumerate_scan_devices(enumerate);
+    devices = udev_enumerate_get_list_entry(enumerate);
+    udev_list_entry_foreach(device, devices) {
+        const char *syspath = udev_list_entry_get_name(device);
+        struct udev_device *udev_device = udev_device_new_from_syspath(udev, syspath);
+        const char *path = udev_device_get_devnode(udev_device);
+        const char *sysname = udev_device_get_sysname(udev_device);
+
+        if (!path || !syspath)
+            goto no_probe;
+        else if (strcmp(udev_device_get_subsystem(udev_device), "drm") != 0)
+            goto no_probe;
+        else if (strncmp(sysname, "card", 4) != 0)
+            goto no_probe;
+
+        config_udev_odev_setup_attribs(path, syspath, probe_callback);
+
+    no_probe:
+        udev_device_unref(udev_device);
+    }
+    udev_enumerate_unref(enumerate);
+    return;
+}
+#endif
+
